
import java.io.*;
import java.util.*;

%%

%public
%class PropertiesReader5
%8bit
%int
%apiprivate

%{

/**
 * PropertiesReader5
 */

	private File file;
	private String comment, key, map, index;
	private StringBuffer value;
	private HashMap bindings;
	private boolean structured; 

	/** Creates a PropertiesReader from a file */
	public PropertiesReader5 (File file) {
		this.file = file;
		value = new StringBuffer();
		bindings = new HashMap();
	}

	/** Creates a PropertiesReader from a filename */
	public PropertiesReader5 (String fileName) {
		this(new File(fileName));
	}

	/** Can be used to run PropertiesReader from the command line. The file to be read is apssed is as parameter. */
	public static void main (String args[]) throws Exception {
		if (args == null) {
			System.err.println("usage: java PropertiesReader5 file");
			System.exit(1);
		}
		PropertiesReader5 self = new PropertiesReader5(args[0]);
		self.importFile();
		
		System.out.println(self.bindings);
	}

	/** Imports the Properties file. */
	public void importFile() {
		try {
			zzReader = new InputStreamReader(new FileInputStream(file));
			//zzReader = new FileReader(fileName);
			while ( !zzAtEOF ) yylex();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

// internal methods follow

	protected void setComment (String comment) {
		this.comment = comment;
		System.out.println("comment="+comment);
	}

	protected void setKey (String key) {
		this.key = key;
		structured = false;
	}

	protected void setMapKey (String map, String index) {
		this.map = map;
		this.index = index;
		structured = true;
	}

	protected void appendValue (String value) {
		this.value.append(value);
	}

	protected void valueDone() {
		if (structured) {
			Object tmp = bindings.get(map);
			if (tmp == null)
				bindings.put(map, new HashMap());
			((Map) bindings.get(map)).put(index, value.toString());
		} else {
			bindings.put(key, value.toString());
		}
	}

// interesting stuff follows below

%}

StartComment = #
WhiteSpace = [ \t]
LineEnding = \r|\n|\r\n

%state COMMENT
%state KEY
%state VALUE

%%

<YYINITIAL> {
	{WhiteSpace}	{
						// ignore
					}

	{StartComment}	{
						yybegin(COMMENT);
					}

	{LineEnding}	{
						// ignore
					}

	.		{
				yypushback(1);
				yybegin(KEY);
			}
}

<COMMENT> {
	.*		{
				setComment(yytext());
			}

	{LineEnding}	{
						yybegin(YYINITIAL);
					}
}

<KEY> {
	[^=]*\.[0-9]+	{
						// this section implements the "hierarchical properties" feature
						// that's not compatible with standard Java properties
						String tmp = yytext();
						setMapKey(tmp.substring(0, tmp.lastIndexOf(".")), tmp.substring(tmp.lastIndexOf(".")+1));
						value.setLength(0);
					}

	[^=]*	{
				setKey(yytext());
				value.setLength(0);
			}

	=		{
				yybegin(VALUE);
			}
}

<VALUE> {
	.*\\{WhiteSpace}*{LineEnding} {
			String tmp = yytext();
			// remove everything from the backslash onwards
			appendValue(tmp.substring(0, tmp.lastIndexOf("\\")));
		}

	.*	{
			appendValue(yytext());
		}

	{LineEnding}	{
						valueDone();
						yybegin(YYINITIAL);
					}
}

